@@ -12,16 +12,19 @@
 #' Ignored if NULL. Otherwise, data transformed before model is estimated.
 #' @param models A character string of up to five characters indicating which contributing models to use:
 #' a (\code{\link[forecast]{auto.arima}}), e (\code{\link[forecast]{ets}}), n (\code{\link[forecast]{nnetar}}),
-#' s (\code{\link[forecast]{stlm}}) and t (\code{\link[forecast]{tbats}})
+#' s (\code{\link[forecast]{stlm}}) and t (\code{\link[forecast]{tbats}}).
 #' @param a.args an optional \code{list} of arguments to pass to \code{\link[forecast]{auto.arima}}. See details.
 #' @param e.args an optional \code{list} of arguments to pass to \code{\link[forecast]{ets}}. See details.
 #' @param n.args an optional \code{list} of arguments to pass to \code{\link[forecast]{nnetar}}. See details.
 #' @param s.args an optional \code{list} of arguments to pass to \code{\link[forecast]{stlm}}. See details.
 #' @param t.args an optional \code{list} of arguments to pass to \code{\link[forecast]{tbats}}. See details.
 #' @param weights method for weighting the forecasts of the various contributing
-#' models.  Defaults to \code{equal}, which has shown to be robust and surprisingly better
-#' in many cases than giving more weight to models with better in-sample performance. Weights
-#' utilizing in-sample errors are also available. Cross validated errors are currently unimplemented.
+#' models.  Defaults to \code{equal}, which has shown to be robust and better
+#' in many cases than giving more weight to models with better in-sample performance. Cross validated errors--implemented with \code{link{cvts}}
+#' should produce the best forecast, but the model estimation is also the slowest. Note that extra arguments
+#' passed in \code{a.args}, \code{e.args}, \code{n.args}, \code{s.args}, and \code{t.args} are not used
+#' during cross validation. See further explanation in \code{\link{cvts}}.
+#' Weights utilizing in-sample errors are also available but not recommended.
 #' @param errorMethod  method of measuring accuracy to use if weights are not
 #' to be equal.
 #' Root mean square error (\code{RMSE}), mean absolute error (\code{MAE})
@@ -31,6 +34,13 @@
 #' This is currently unimplemented.
 #' Parallelization will still occur within individual models that suport it and can be controlled using \code{a.args} and \code{t.args}.
 #' @param num.cores If \code{parallel=TRUE}, how many cores to use.
+#' @param cvHorizon If \code{weights = "cv.errors"}, this controls which forecast to horizon to use
+#' for the error calculations.
+#' @param windowSize length of the window to build each model, only used when \code{weights = "cv.errors"}.
+#' @param horizonAverage If \code{weights = "cv.errors"}, setting this to \code{TRUE} will average
+#' all forecast horizons up to \code{cvHorizon} for calculating the errors instead of using
+#' the single horizon given in \code{cvHorizon}.
+#' @param verbose Should the status of which model is being fit/cross validated be printed to the terminal?
 #' @seealso \code{\link{forecast.hybridModel}}, \code{\link[forecast]{auto.arima}},
 #' \code{\link[forecast]{ets}}, \code{\link[forecast]{nnetar}},
 #' \code{\link[forecast]{stlm}}, \code{\link[forecast]{tbats}}
@@ -41,11 +51,12 @@
 #' of ensemble forecasts. While default settings for the individual component models work quite well
 #' in most cases, fine control can be exerted by passing detailed arguments to the component models in the
 #' \code{a.args}, \code{e.args}, \code{n.args}, \code{s.args}, and \code{t.args} lists.
-#' Note that if \code{xreg} is passed to the \code{a.args} or \code{n.args} component models
+#' Note that if \code{xreg} is passed to the \code{a.args}, \code{n.args}, or \code{s.args} component models
 #' it must be passed as a dataframe instead of the matrix object
 #' that the "forecast" package functions usually accept.
 #' This is due to a limitation in how the component models are called.
 #' \cr
+#' \cr
 #' Characteristics of the input series can cause problems for certain types of models and paramesters.
 #' For example, \code{\link[forecast]{stlm}} models require that the input series be seasonal;
 #' furthemore, the data must include at least two seasons of data (i.e. \code{length(y) >= 2 * frequency(y)})
@@ -61,17 +72,27 @@
 #' @examples
 #' \dontrun{
 #'
+#' # Fit an auto.arima, ets, nnetar, stlm, and tbats model
+#' # on the time series with equal weights
 #' mod1 <- hybridModel(AirPassengers)
 #' plot(forecast(mod1))
+#'
+#' # Use an auto.arima, ets, and tbats model with weights
+#' # set by the MASE in-sample errors
 #' mod2 <- hybridModel(AirPassengers, models = "aet",
 #' weights = "insample.errors", errorMethod = "MASE")
+#'
+#' # Pass additional arguments to auto.arima() to control its fit
 #' mod3 <- hybridModel(AirPassengers, models = "aens",
 #' a.args = list(max.p = 7, max.q = 7, approximation = FALSE))
+#'
 #' # View the component auto.arima() and stlm() models
 #' mod3$auto.arima
 #' mod3$stlm
 #' }
 #'
+#' @author David Shaub
+#'
 hybridModel <- function(y, models = "aenst",
                         lambda = NULL,
                         a.args = NULL,
@@ -81,7 +102,11 @@ hybridModel <- function(y, models = "aenst",
                         t.args = NULL,
                         weights = c("equal", "insample.errors", "cv.errors"),
                         errorMethod = c("RMSE", "MAE", "MASE"),
-                        parallel = FALSE, num.cores = 2L){
+                        cvHorizon = frequency(y),
+                        windowSize = 84,
+                        horizonAverage = FALSE,
+                        parallel = FALSE, num.cores = 2L,
+                        verbose = TRUE){
   # Weights could be set to equal (the default), based on in-sample errors, or based on cv errors
   # errorMethod will determine which type of errors to use for weights. Some choices from accuracy()
   # are not appropriate. If weights = "equal", this would be ignored.
@@ -151,15 +176,15 @@ hybridModel <- function(y, models = "aenst",
       expandedModels <- expandedModels[expandedModels != "s"]
     }
     if(frequency(y) * 2L >= length(y)){
-       warning("The stlm model requres a series more than twice as long as the seasonal period. The stlm model will not be used.")
-       expandedModels <- expandedModels[expandedModels != "s"]
+      warning("The stlm model requres a series more than twice as long as the seasonal period. The stlm model will not be used.")
+      expandedModels <- expandedModels[expandedModels != "s"]
     }
   }
   if(is.element("n", expandedModels)){
-     if(frequency(y) * 2L >= length(y)){
-        warning("The nnetar model requres a series more than twice as long as the seasonal period. The nnetar model will not be used.")
-        expandedModels <- expandedModels[expandedModels != "n"]
-     }
+    if(frequency(y) * 2L >= length(y)){
+      warning("The nnetar model requres a series more than twice as long as the seasonal period. The nnetar model will not be used.")
+      expandedModels <- expandedModels[expandedModels != "n"]
+    }
   }
 
 
@@ -172,9 +197,13 @@ hybridModel <- function(y, models = "aenst",
   if(parallel){
     warning("The 'parallel' argument is currently unimplemented. Ignoring for now.")
   }
-  if(weights == "cv.errors"){
-    warning("Cross validated error weights are currently unimplemented. Ignoring for now.")
-     weights <- "equal"
+  # if(weights == "cv.errors"){
+  #   warning("Cross validated error weights are currently unimplemented. Ignoring for now.")
+  #   weights <- "equal"
+  # }
+  if(weights == "cv.errors" && errorMethod == "MASE"){
+    warning("cv errors currently do not support MASE. Reverting to RMSE.")
+    errorMethod <- "RMSE"
   }
 
   modelResults <- list()
@@ -184,6 +213,9 @@ hybridModel <- function(y, models = "aenst",
   # parallelism both within and between models, based on the number of available cores.
   # auto.arima()
   if(is.element("a", expandedModels)){
+    if(verbose){
+      cat("Fitting the auto.arima model\n")
+    }
     if(is.null(a.args)){
       a.args <- list(lambda = lambda)
     } else if(is.null(a.args$lambda)){
@@ -193,6 +225,9 @@ hybridModel <- function(y, models = "aenst",
   }
   # ets()
   if(is.element("e", expandedModels)){
+    if(verbose){
+      cat("Fitting the ets model\n")
+    }
     if(is.null(e.args)){
       e.args <- list(lambda = lambda)
     } else if(is.null(e.args$lambda)){
@@ -202,6 +237,9 @@ hybridModel <- function(y, models = "aenst",
   }
   # nnetar()
   if(is.element("n", expandedModels)){
+    if(verbose){
+      cat("Fitting the nnetar model\n")
+    }
     if(is.null(n.args)){
       n.args <- list(lambda = lambda)
     } else if(is.null(n.args$lambda)){
@@ -211,6 +249,9 @@ hybridModel <- function(y, models = "aenst",
   }
   # stlm()
   if(is.element("s", expandedModels)){
+    if(verbose){
+      cat("Fitting the stlm model\n")
+    }
     if(is.null(s.args)){
       s.args <- list(lambda = lambda)
     } else if(is.null(s.args$lambda)){
@@ -220,6 +261,9 @@ hybridModel <- function(y, models = "aenst",
   }
   # tbats()
   if(is.element("t", expandedModels)){
+    if(verbose){
+      cat("Fitting the tbats model\n")
+    }
     modelResults$tbats <- do.call(tbats, c(list(y), e.args))
   }
 
@@ -228,22 +272,81 @@ hybridModel <- function(y, models = "aenst",
   # Weighting methods would go here, equal weighting for now
   if(weights == "equal"){
     modelResults$weights <- rep(1 / length(expandedModels), length(expandedModels))
-  } else if(weights == "insample.errors"){
+  } else if(weights %in% c("insample.errors", "cv.errors")){
+
     # There is probably a better way of accomplishing this
     # But this ugly approach will work for now
+    # These loops and if statements can be replace
+    # with do.call and/or map
     modelResults$weights <- rep(0, length(expandedModels))
     index <- 1
+    modResults <- modelResults
+
+    if(weights == "cv.errors"){
+      #modResults <-
+      for(i in expandedModels){
+        if(i == "a"){
+          if(verbose){
+            cat("Cross validating the auto.arima model\n")
+          }
+          modResults$auto.arima <- cvts(y, FUN = auto.arima,
+                                        maxHorizon = cvHorizon,
+                                        horizonAverage = horizonAverage,
+                                        verbose = FALSE,
+                                        windowSize = windowSize)
+        } else if(i == "e"){
+          if(verbose){
+            cat("Cross validating the ets model\n")
+          }
+          modResults$ets <- cvts(y, FUN = ets,
+                                 maxHorizon = cvHorizon,
+                                 horizonAverage = horizonAverage,
+                                 verbose = FALSE,
+                                 windowSize = windowSize)
+        } else if(i == "n"){
+          if(verbose){
+            cat("Cross validating the nnetar model\n")
+          }
+          modResults$nnetar <- cvts(y, FUN = nnetar,
+                                    maxHorizon = cvHorizon,
+                                    horizonAverage = horizonAverage,
+                                    verbose = FALSE,
+                                    windowSize = windowSize)
+        } else if(i == "s"){
+          if(verbose){
+            cat("Cross validating the stlm model\n")
+          }
+          modResults$stlm <- cvts(y, FUN = stlm,
+                                  maxHorizon = cvHorizon,
+                                  horizonAverage = horizonAverage,
+                                  verbose = FALSE,
+                                  windowSize = windowSize)
+        } else if(i == "t"){
+          if(verbose){
+            cat("Cross validating the tbats model\n")
+          }
+          modResults$tbats <- cvts(y, FUN = tbats,
+                                   maxHorizon = cvHorizon,
+                                   horizonAverage = horizonAverage,
+                                   verbose = FALSE,
+                                   windowSize = windowSize)
+        }
+      }
+    }
+    # If horizonAverage == TRUE, the resulting accuracy object will have only one row
+    cvHorizon <- ifelse(horizonAverage, 1, cvHorizon)
+    cvHorizon <- ifelse(weights != "cv.errors", 1, cvHorizon)
     for(i in expandedModels){
       if(i == "a"){
-        modelResults$weights[index] <- accuracy(modelResults$auto.arima)[1, errorMethod]
+        modelResults$weights[index] <- accuracy(modResults$auto.arima)[cvHorizon, errorMethod]
       }else if(i == "e"){
-        modelResults$weights[index] <- accuracy(modelResults$ets)[1, errorMethod]
+        modelResults$weights[index] <- accuracy(modResults$ets)[cvHorizon, errorMethod]
       } else if(i == "n"){
-        modelResults$weights[index] <- accuracy(modelResults$nnetar)[1, errorMethod]
+        modelResults$weights[index] <- accuracy(modResults$nnetar)[cvHorizon, errorMethod]
       } else if(i == "s"){
-        modelResults$weights[index] <- accuracy(modelResults$stlm)[1, errorMethod]
+        modelResults$weights[index] <- accuracy(modResults$stlm)[cvHorizon, errorMethod]
       } else if(i == "t"){
-        modelResults$weights[index] <- accuracy(modelResults$tbats)[1, errorMethod]
+        modelResults$weights[index] <- accuracy(modResults$tbats)[cvHorizon, errorMethod]
       }
       index <- index + 1
     }
@@ -251,6 +354,12 @@ hybridModel <- function(y, models = "aenst",
     modelResults$weights <- (1 / modelResults$weights) / sum(1 / modelResults$weights)
 
   }
+
+  # Check for valid weights when weights = "insample.errors" and submodels produce perfect fits
+  if(is.element(NaN, modelResults$weights) & weights %in% c("insample.errors", "cv.errors")){
+    warning('At least one model perfectly fit the series, so accuracy measures cannot be used for weights. Reverting to weights = "equal".')
+    modelResults$weights <- rep(1/ length(includedModels), length(includedModels))
+  }
   names(modelResults$weights) <- includedModels
 
   # Apply the weights to construct the fitted values
@@ -277,6 +386,8 @@ hybridModel <- function(y, models = "aenst",
 
 #' Test if the object is a hybridModel object
 #'
+#' Test if the object is a \code{hybridModel} object.
+#'
 #' @export
 #' @param x the input object.
 #' @return A boolean indicating if the object is a \code{hybridModel} is returned.
@@ -300,6 +411,7 @@ is.hybridModel <- function(x){
 fitted.hybridModel <- function(object,
                                individual = FALSE,
                                ...){
+  #chkDots(...)
   if(individual){
     results <- list()
     for(i in object$models){
@@ -314,7 +426,7 @@ fitted.hybridModel <- function(object,
 #'
 #' Extract the model residuals from the \code{hybridModel} object.
 #' @export
-#' @param object The input hybridModel
+#' @param object The input hybridModel.
 #' @param individual If \code{TRUE}, return the residuals of the component models instead
 #' of the residuals for the whole ensemble model.
 #' @param ... Other arguments (ignored).
@@ -324,6 +436,7 @@ fitted.hybridModel <- function(object,
 residuals.hybridModel <- function(object,
                                   individual = FALSE,
                                   ...){
+  #chkDots(...)
   if(individual){
     results <- list()
     for(i in object$models){
@@ -337,19 +450,23 @@ residuals.hybridModel <- function(object,
 
 #' Generic method for accuracy
 #'
+#' Generic method for accuracy.
+#'
 #' @param f an object of class forecast, or a numerical vector containing forecasts.
 #' It will also work with Arima, ets and lm objects if x is omitted - in which case
 #' in-sample accuracy measures are returned.
-#' @param ... other arguments (ignored)
+#' @param ... other arguments (ignored).
 #' @seealso \code{\link[forecast]{accuracy}}, \code{\link{accuracy.hybridModel}}
 #' @export
 #'
 accuracy.default <-  function(f, ...){
-   forecast::accuracy(f, ...)
+  forecast::accuracy(f, ...)
 }
 
 #' Generic method for accuracy
 #'
+#' Generic method for accuracy.
+#'
 #' @param f the input object.
 #' @param ... other arguments (ignored).
 #' @seealso \code{\link[forecast]{accuracy}}, \code{\link{accuracy.hybridModel}}
@@ -360,6 +477,9 @@ accuracy <- function(f,...){
 
 #' Accuracy measures for hybridModel objects
 #'
+#' Accuracy measures for hybridModel
+#' objects.
+#'
 #' Return the in-sample accuracy measures for the component models of the hybridModel
 #'
 #' @param f the input hybridModel.
@@ -370,9 +490,12 @@ accuracy <- function(f,...){
 #' @return The accuracy of the ensemble or individual component models.
 #' @export
 #'
+#' @author David Shaub
+#'
 accuracy.hybridModel <- function(f,
                                  individual = FALSE,
                                  ...){
+  #chkDots(...)
   if(individual){
     results <- list()
     for(i in f$models){
@@ -383,29 +506,58 @@ accuracy.hybridModel <- function(f,
   return(forecast::accuracy(f$fitted, getResponse(f)))
 }
 
+#'Accuracy measures for cross-validated time series
+#'
+#'Returns range of summary measures of the cross-validated forecast accuracy
+#'for \code{cvts} objects.
+#'
+#'@param f a \code{cvts} objected created by \code{\link{cvts}}.
+#'@param ... other arguments (ignored).
+#'
+#'@details
+#'Currently the method only implements \code{ME}, \code{RMSE}, and \code{MAE}. The accuracy measures
+#'\code{MPE}, \code{MAPE}, and \code{MASE} are not calculated. The accuracy is calculated for each
+#'forecast horizon up to \code{maxHorizon}
+#'
+accuracy.cvts <- function(f, ...){
+  ME <- colMeans(f$residuals)
+  RMSE <- apply(f$residuals, MARGIN = 2, FUN = function(x){sqrt(sum(x ^ 2)/ length(x))})
+  MAE <- colMeans(abs(f$residuals))
+  results <- data.frame(ME, RMSE, MAE)
+  rownames(results) <- paste("Forecast Horizon ", rownames(results))
+  return(results)
+  # MASE TODO
+  # Will require actual/fitted/residuals
+}
+
+
 #' Print a summary of the hybridModel object
 #'
 #' @param x the input \code{hybridModel} object.
 #' @details Print the names of the individual component models and their weights.
 #'
+#'
 summary.hybridModel <- function(x){
   print(x)
 }
 
 #' Print information about the hybridModel object
 #'
+#' Print information about the \code{hybridModel} object.
+#'
 #' @param x the input \code{hybridModel} object.
 #' @param ... other arguments (ignored).
 #' @export
 #' @details Print the names of the individual component models and their weights.
 #'
 print.hybridModel <- function(x, ...){
+  #chkDots(...)
   cat("Hybrid forecast model comprised of the following models: ")
   cat(x$models, sep = ", ")
   cat("\n")
   for(i in x$models){
     cat("############\n")
-    cat(i, "with weight", x$weights[i], "\n")
+    cat(i, "with weight", round(x$weights[i], 3), "\n")
     #cat("############\n")
   }
 }
@@ -422,10 +574,11 @@ print.hybridModel <- function(x, ...){
 #' \code{\link[forecast]{plot.Arima}}, \code{\link[forecast]{plot.ets}},
 #' \code{\link[forecast]{plot.tbats}}. Note: no plot
 #' methods exist for \code{nnetar} and \code{stlm} objects, so these will not be plotted with
-#' \code{type = "models"}
-#' @param ... other arguments (ignored).
+#' \code{type = "models"}.
+#' @param ... other arguments passed to \link{plot}.
 #' @seealso \code{\link{hybridModel}}
 #' @return None. Function produces a plot.
+#'
 #' @details For \code{type = "fit"}, the original series is plotted in black.
 #' Fitted values for the individual component models are plotted in other colors.
 #' For \code{type = "models"}, each individual component model is plotted. Since
@@ -439,19 +592,21 @@ print.hybridModel <- function(x, ...){
 #' }
 #' @export
 #'
+#' @author David Shaub
+#'
 plot.hybridModel <- function(x,
                              type = c("fit", "models"),
                              ...){
   type <- match.arg(type)
+  #chkDots(...)
   plotModels <- x$models
   if(type == "fit"){
     # Set the highest and lowest axis scale
     ymax <- max(sapply(plotModels, FUN = function(i) max(fitted(x[[i]]), na.rm = TRUE)))
     ymin <- min(sapply(plotModels, FUN = function(i) min(fitted(x[[i]]), na.rm = TRUE)))
     range <- ymax - ymin
-    plot(x$x, ylim = c(ymin - 0.05 * range, ymax + 0.25 * range),
-         ylab = "y", xlab = "time")
-    title(main = "Plot of original series (black) and fitted component models", outer = TRUE)
+    plot(x$x, ylim = c(ymin - 0.05 * range, ymax + 0.25 * range), ...)
+    #title(main = "Plot of original series (black) and fitted component models", outer = TRUE)
     for(i in seq_along(plotModels)){
       lines(fitted(x[[plotModels[i]]]), col = i + 1)
     }
